==> ember-1.7.tar.gz: ember-1.7/src/libember/www.c <==
/*
**      www.c - 
**
**
** Copyright (c) 2001  Hughes Technologies Pty Ltd
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** This software is provided "as is" without any expressed or implied warranty.
**
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>

#include "config/config.h"
#include "regexp/regexp.h"
#include "ember.h"

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif


/*************************************************************************
** URL encoding / decoding routines.
**
*/

static char six2pr[64] = {
    'A','B','C','D','E','F','G','H','I','J','K','L','M',
    'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
    'a','b','c','d','e','f','g','h','i','j','k','l','m',
    'n','o','p','q','r','s','t','u','v','w','x','y','z',
    '0','1','2','3','4','5','6','7','8','9','+','/'   
};
  
static unsigned char pr2six[256];

int eWwwDecode (bufcoded, bufplain, outbufsize)
	char *		bufcoded;
	char *		bufplain;
	int		outbufsize;
{
/* single character decode */
#define DEC(c) pr2six[(int)c]
#define MAXVAL 63

   static int first = 1;

   int nbytesdecoded, j;
   register char *bufin = bufcoded;
   register unsigned char *bufout = bufplain;
   register int nprbytes;

   /* If this is the first call, initialize the mapping table.
    * This code should work even on non-ASCII machines.
    */
   if(first) {
      first = 0;
      for(j=0; j<256; j++) pr2six[j] = MAXVAL+1;
      for(j=0; j<64; j++) pr2six[(int)six2pr[j]] = (unsigned char) j;
   }

   /* Strip leading whitespace. */

   while(*bufcoded==' ' || *bufcoded == '\t') bufcoded++;

   /* Figure out how many characters are in the input buffer.
    * If this would decode into more bytes than would fit into
    * the output buffer, adjust the number of input bytes downwards.
    */
   bufin = bufcoded;
   while(pr2six[(int)*(bufin++)] <= MAXVAL);
   nprbytes = bufin - bufcoded - 1;
   nbytesdecoded = ((nprbytes+3)/4) * 3;
   if(nbytesdecoded > outbufsize) {
      nprbytes = (outbufsize*4)/3;
   }

   bufin = bufcoded;
   
   while (nprbytes > 0) {
      *(bufout++) = (unsigned char) (DEC(*bufin) << 2 | DEC(bufin[1]) >> 4);
      *(bufout++) = (unsigned char) (DEC(bufin[1]) << 4 | DEC(bufin[2]) >> 2);
      *(bufout++) = (unsigned char) (DEC(bufin[2]) << 6 | DEC(bufin[3]));
      bufin += 4;
      nprbytes -= 4;
   }
   
   if(nprbytes & 03) {
      if(pr2six[(int)bufin[-2]] > MAXVAL) {
         nbytesdecoded -= 2;
      } else {
         nbytesdecoded -= 1;
      }
   }

   return(nbytesdecoded);
}

static char from_hex (c)
        char    c;
{
    return  c >= '0' && c <= '9' ?  c - '0'
            : c >= 'A' && c <= 'F'? c - 'A' + 10
            : c - 'a' + 10;     /* accept small letters just in case */
}

char * eWwwUnEscape (str)
        char    *str;
{
    char * p = str;
    char * q = str;
    static char blank[] = "";

    if (!str)
        return(blank);
    while(*p) {
        if (*p == '%') {
            p++;
            if (*p) *q = from_hex(*p++) * 16;
            if (*p) *q = (*q + from_hex(*p++));
            q++;
        } else {
            if (*p == '+') {
                *q++ = ' ';
                p++;
            } else {
                *q++ = *p++;
              }
        }
    }

    *q++ = 0;
    return str;
} /* HTUnEscape */


/***********************************************************************
**
** End of encoding routines.  On with the show
**
*/


void eWwwAddHeader(script, header)
	ember	*script;
	char	*header;
{
        if (script->www.customHeader)
        {
		script->www.customHeader=realloc(script->www.customHeader,
                        strlen(script->www.customHeader) +
                        strlen(header) + 2);
        }
        else
        {
                script->www.customHeader=malloc(strlen(header) + 2);
                * script->www.customHeader = 0;
        }
        strcat(script->www.customHeader, header);
        strcat(script->www.customHeader,"\n");
}


int eWwwSendResponse(script, resp)
	ember	*script;
	char	*resp;
{
	static	char	standard[] = "200 Output follows";
	char	*output;


	/*
	** Bail out if we've already done this
	*/
	if (script->www.responseSent)
	{
		return(-1);
	}

	/*
	** Ok, if we are given a response, use it.  If not, look for
	** one in the script structure.  If that's not there then just
	** use the standard one.
	*/
	output = NULL;
	if (resp)
	{
		output = resp;
	}

	if (output == NULL && script->www.customResponse != NULL)
	{
		output = script->www.customResponse;
	}
	if (output == NULL)
	{
		output = standard;
	}
	if (output[strlen(output) - 1] == '\n')
		output[strlen(output) - 1] = 0;
	printf("Status: %s\n",output);
	script->www.responseSent = 1;
	return(0);
}


int eWwwSendHeaders(script)
	ember	*script;
{
	if (script->www.headersSent)
	{
		return(-1);
	}
	if (script->www.contentType)
	{
		printf("Content-Type: %s\n",script->www.contentType);
	}
	else
	{
		printf("Content-Type: text/html\n");
	}
	printf("Pragma: No-Cache\n");
	/*printf("Cache-Control: no-cache\n");*/
	if (script->www.customHeader)
	{
		printf(script->www.customHeader);
	}
	printf("\n");
	script->www.headersSent = 1;
	return(0);
}



void eWwwSendAuthHeader(script, realm, msg, file)
	ember	*script;
        char    *realm,
                *msg,
                *file;
{
        int     fd, 
                len;
        char    buf[1024];

	if(eWwwSendResponse(script, "401 Error") < 0)
		return;
        sprintf(buf, "WWW-Authenticate: Basic realm=\" '%s' \"\n",realm);
	eWwwAddHeader(script, buf);
	eWwwSendHeaders(script);
	fflush(stdout);
	if (!file)
	{
		printf("<BODY BGCOLOR=#FFFFFF TEXT=#0606A0 LINK=#0000FF ");
		printf(" VLINK=#0000FF>\n");
		printf("<P><BR><CENTER>");
		printf("<IMG SRC=/icons/alert.red.gif><P><BR>\n");
		printf("<H1><I>Access Denied</I></H1>\n");
		printf("<P><BR><H3>%s</H3></CENTER><P>\n",msg);
	}
	else
	{
		fd = open(file, O_RDONLY, 0);
		if (fd < 0)
		{
			printf("<H2>Authentication failed</H2>\n");
		}
		else
		{
			len = read(fd,buf,1024);
			while(len > 0)
			{
				write(fileno(stdout),buf,len);
				len = read(fd,buf,1024);
			}
			close(fd);
		}
	}
}



void eWwwAuth(script, uname, passwd, area, file)
	ember	*script;
        char    *uname,
                *passwd,
                *area,
                *file;
{
        char    *authInfo,
                authBuf[100],
                *tok;
        int     length;

        /*
        ** Get the username & passwd
        */
        authInfo = (char *)getenv("HTTP_AUTHORIZATION");
        if (!authInfo)
        {
                eWwwSendAuthHeader(script, area, 
			"Missing Authentication Details",file);
                exit(1);
        }
        authInfo = (char *)index(authInfo,' ');
        if (!authInfo)
        {
                eWwwSendAuthHeader(script, area, 
			"Malformed Authentication Details",file);
                exit(1);
        }
        authInfo++;
        bzero(authBuf,100);
        length = eWwwDecode(authInfo, authBuf, 100);
        *(authBuf+length) = 0;
        tok = (char *)strtok(authBuf,":");
        if (tok)
                strcpy(uname,tok);
        else
                *uname = 0;
        tok = (char *)strtok(NULL,"\n");
        if (tok)
                strcpy(passwd,tok);
        else
                *passwd = 0;
        if (!(*uname) || !(*passwd))
        {
                eWwwSendAuthHeader(script, area, 
			"Missing Username or Password",file);
                exit(1);
        }
}

==> ember-1.8.tar.gz: ember-1.8/src/libember/www.c <==
/*
**      www.c - 
**
**
** Copyright (c) 2001  Hughes Technologies Pty Ltd
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** This software is provided "as is" without any expressed or implied warranty.
**
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>

#include "config/config.h"
#include "regexp/regexp.h"
#include "ember.h"

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif


/*************************************************************************
** URL encoding / decoding routines.
**
*/

static char six2pr[64] = {
    'A','B','C','D','E','F','G','H','I','J','K','L','M',
    'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
    'a','b','c','d','e','f','g','h','i','j','k','l','m',
    'n','o','p','q','r','s','t','u','v','w','x','y','z',
    '0','1','2','3','4','5','6','7','8','9','+','/'   
};
  
static unsigned char pr2six[256];

void eWwwOutput(script, str, len)
	ember	*script;
	char	*str;
	int	len;
{
#ifdef EMBER_API_VER2
	write(script->stdoutFD, str, len);
#else
	write(fileno(stdout), str, len);
#endif
}



int eWwwDecode (bufcoded, bufplain, outbufsize)
	char *		bufcoded;
	char *		bufplain;
	int		outbufsize;
{
/* single character decode */
#define DEC(c) pr2six[(int)c]
#define MAXVAL 63

   static int first = 1;

   int nbytesdecoded, j;
   register char *bufin = bufcoded;
   register unsigned char *bufout = bufplain;
   register int nprbytes;

   /* If this is the first call, initialize the mapping table.
    * This code should work even on non-ASCII machines.
    */
   if(first) {
      first = 0;
      for(j=0; j<256; j++) pr2six[j] = MAXVAL+1;
      for(j=0; j<64; j++) pr2six[(int)six2pr[j]] = (unsigned char) j;
   }

   /* Strip leading whitespace. */

   while(*bufcoded==' ' || *bufcoded == '\t') bufcoded++;

   /* Figure out how many characters are in the input buffer.
    * If this would decode into more bytes than would fit into
    * the output buffer, adjust the number of input bytes downwards.
    */
   bufin = bufcoded;
   while(pr2six[(int)*(bufin++)] <= MAXVAL);
   nprbytes = bufin - bufcoded - 1;
   nbytesdecoded = ((nprbytes+3)/4) * 3;
   if(nbytesdecoded > outbufsize) {
      nprbytes = (outbufsize*4)/3;
   }

   bufin = bufcoded;
   
   while (nprbytes > 0) {
      *(bufout++) = (unsigned char) (DEC(*bufin) << 2 | DEC(bufin[1]) >> 4);
      *(bufout++) = (unsigned char) (DEC(bufin[1]) << 4 | DEC(bufin[2]) >> 2);
      *(bufout++) = (unsigned char) (DEC(bufin[2]) << 6 | DEC(bufin[3]));
      bufin += 4;
      nprbytes -= 4;
   }
   
   if(nprbytes & 03) {
      if(pr2six[(int)bufin[-2]] > MAXVAL) {
         nbytesdecoded -= 2;
      } else {
         nbytesdecoded -= 1;
      }
   }

   return(nbytesdecoded);
}

static char from_hex (c)
        char    c;
{
    return  c >= '0' && c <= '9' ?  c - '0'
            : c >= 'A' && c <= 'F'? c - 'A' + 10
            : c - 'a' + 10;     /* accept small letters just in case */
}

char * eWwwUnEscape (str)
        char    *str;
{
    char * p = str;
    char * q = str;
    static char blank[] = "";

    if (!str)
        return(blank);
    while(*p) {
        if (*p == '%') {
            p++;
            if (*p) *q = from_hex(*p++) * 16;
            if (*p) *q = (*q + from_hex(*p++));
            q++;
        } else {
            if (*p == '+') {
                *q++ = ' ';
                p++;
            } else {
                *q++ = *p++;
              }
        }
    }

    *q++ = 0;
    return str;
} /* HTUnEscape */


/***********************************************************************
**
** End of encoding routines.  On with the show
**
*/


void eWwwAddHeader(script, header)
	ember	*script;
	char	*header;
{
        if (script->www.customHeader)
        {
		script->www.customHeader=realloc(script->www.customHeader,
                        strlen(script->www.customHeader) +
                        strlen(header) + 2);
        }
        else
        {
                script->www.customHeader=malloc(strlen(header) + 2);
                * script->www.customHeader = 0;
        }
        strcat(script->www.customHeader, header);
        strcat(script->www.customHeader,"\n");
}


int eWwwSendResponse(script, resp)
	ember	*script;
	char	*resp;
{
	static	char	standard[] = "200 Output follows";
	char	*output;


	/*
	** Bail out if we've already done this
	*/
	if (script->www.responseSent)
	{
		return(-1);
	}

	/*
	** Ok, if we are given a response, use it.  If not, look for
	** one in the script structure.  If that's not there then just
	** use the standard one.
	*/
	output = NULL;
	if (resp)
	{
		output = resp;
	}

	if (output == NULL && script->www.customResponse != NULL)
	{
		output = script->www.customResponse;
	}
	if (output == NULL)
	{
		output = standard;
	}
	if (output[strlen(output) - 1] == '\n')
		output[strlen(output) - 1] = 0;

#ifdef EMBER_API_VER2
	if (script->www.rawHTTP)
	{
		eWwwOutput(script, "HTTP/1.0 ", 9);
	}
	else
	{
		eWwwOutput(script, "Status: ", 8);
	}
#else
	eWwwOutput(script, "Status:", 8);
#endif
	eWwwOutput(script, output, strlen(output));
	eWwwOutput(script, "\n", 1);
	script->www.responseSent = 1;
	return(0);
}


int eWwwSendHeaders(script)
	ember	*script;
{
	static	char outBuf[160];

	if (script->www.headersSent)
	{
		return(-1);
	}
	if (script->www.contentType)
	{
		snprintf(outBuf,160,"Content-Type: %s\n",
			script->www.contentType);
	}
	else
	{
		strcpy(outBuf,"Content-Type: text/html\n");
	}
	eWwwOutput(script, outBuf, strlen(outBuf));
	strcpy(outBuf,"Pragma: No-Cache\n");
	eWwwOutput(script, outBuf, strlen(outBuf));
	if (script->www.customHeader)
	{
		eWwwOutput(script, script->www.customHeader,
			strlen(script->www.customHeader));
	}
	eWwwOutput(script, "\n",1);
	script->www.headersSent = 1;
	return(0);
}



void eWwwSendAuthHeader(script, realm, msg, file)
	ember	*script;
        char    *realm,
                *msg,
                *file;
{
        int     fd, 
                len;
        char    buf[1024];

	if(eWwwSendResponse(script, "401 Error") < 0)
		return;
        sprintf(buf, "WWW-Authenticate: Basic realm=\" '%s' \"\n",realm);
	eWwwAddHeader(script, buf);
	eWwwSendHeaders(script);
	fflush(stdout);
	if (!file)
	{
		printf("<BODY BGCOLOR=#FFFFFF TEXT=#0606A0 LINK=#0000FF ");
		printf(" VLINK=#0000FF>\n");
		printf("<P><BR><CENTER>");
		printf("<IMG SRC=/icons/alert.red.gif><P><BR>\n");
		printf("<H1><I>Access Denied</I></H1>\n");
		printf("<P><BR><H3>%s</H3></CENTER><P>\n",msg);
	}
	else
	{
		fd = open(file, O_RDONLY, 0);
		if (fd < 0)
		{
			printf("<H2>Authentication failed</H2>\n");
		}
		else
		{
			len = read(fd,buf,1024);
			while(len > 0)
			{
				write(fileno(stdout),buf,len);
				len = read(fd,buf,1024);
			}
			close(fd);
		}
	}
}



void eWwwAuth(script, uname, passwd, area, file)
	ember	*script;
        char    *uname,
                *passwd,
                *area,
                *file;
{
        char    *authInfo,
                authBuf[100],
                *tok;
        int     length;

        /*
        ** Get the username & passwd
        */
        authInfo = (char *)getenv("HTTP_AUTHORIZATION");
        if (!authInfo)
        {
                eWwwSendAuthHeader(script, area, 
			"Missing Authentication Details",file);
                exit(1);
        }
        authInfo = (char *)index(authInfo,' ');
        if (!authInfo)
        {
                eWwwSendAuthHeader(script, area, 
			"Malformed Authentication Details",file);
                exit(1);
        }
        authInfo++;
        bzero(authBuf,100);
        length = eWwwDecode(authInfo, authBuf, 100);
        *(authBuf+length) = 0;
        tok = (char *)strtok(authBuf,":");
        if (tok)
                strcpy(uname,tok);
        else
                *uname = 0;
        tok = (char *)strtok(NULL,"\n");
        if (tok)
                strcpy(passwd,tok);
        else
                *passwd = 0;
        if (!(*uname) || !(*passwd))
        {
                eWwwSendAuthHeader(script, area, 
			"Missing Username or Password",file);
                exit(1);
        }
}
